@@ -24,9 +24,9 @@
 
 GPid rsync_pid;
 GIOChannel *chout, *cherr;
-gint session_last = -1, initial_width, initial_height, root_x, root_y, width, height;
+gint session_last = -1, session_set = -1, initial_width, initial_height, root_x, root_y, width, height;
 gsize session_number = 0;
-gboolean had_error, paused, first_load_groups = TRUE, dryrunning = FALSE, must_scroll = FALSE;
+gboolean had_error, paused, first_load_groups = TRUE, dryrunning = FALSE, must_scroll = FALSE, is_set = FALSE;
 gchar **groups = NULL, **argv, *error_list = NULL, *watch_oldfile = NULL;
 GtkWidget *rsync_window = NULL;
 gboolean config_output = FALSE, config_remember = TRUE, config_errorlist = FALSE, config_log = FALSE, config_fastscroll = TRUE;
@@ -38,6 +38,7 @@ gchar config_command[MAXPATH], *grsync_dir, *rsync_help = NULL, *rsync_man = NUL
 FILE *log_file = NULL;
 glong scroll_previous_run;
 GtkStatusIcon *trayIcon = NULL;
+GtkTreePath *path_set = NULL;
 
 
 void on_trayicon_activate(GObject *trayIcon, gpointer window) {
@@ -111,6 +112,7 @@ void set_main_title(gchar* session, gchar* extra) {
 
 void load_settings(gchar* session, gchar *filename) {
 	gchar settings_file_path[MAXPATH], *stmp;
+	static gboolean resized = FALSE;
 	GKeyFile *settings_file;
 
 	if (filename == NULL) g_sprintf(settings_file_path, "%s/%s", grsync_dir, "grsync.ini");
@@ -160,64 +162,96 @@ void load_settings(gchar* session, gchar *filename) {
 #endif
 
 		// load window position and size
-		root_x = g_key_file_get_integer(settings_file, CONFIG_GROUP, "root_x", NULL);
-		root_y = g_key_file_get_integer(settings_file, CONFIG_GROUP, "root_y", NULL);
-		width  = g_key_file_get_integer(settings_file, CONFIG_GROUP, "width", NULL);
-		height = g_key_file_get_integer(settings_file, CONFIG_GROUP, "height", NULL);
-		gtk_window_move((GtkWindow*) main_window, root_x, root_y);
-		if (width > 0 && height > 0) gtk_window_resize((GtkWindow*) main_window, width, height);
+		if (!resized) {
+			root_x = g_key_file_get_integer(settings_file, CONFIG_GROUP, "root_x", NULL);
+			root_y = g_key_file_get_integer(settings_file, CONFIG_GROUP, "root_y", NULL);
+			width  = g_key_file_get_integer(settings_file, CONFIG_GROUP, "width", NULL);
+			height = g_key_file_get_integer(settings_file, CONFIG_GROUP, "height", NULL);
+			gtk_window_move((GtkWindow*) main_window, root_x, root_y);
+			if (width > 0 && height > 0) gtk_window_resize((GtkWindow*) main_window, width, height);
+			resized = TRUE;
+		}
 	}
 	set_main_title(session, NULL);		// put here because it should be after set_trayicon()
 
 	// load specific session
-	ini_set_entry(settings_file, session, "text_source", "text_source");
-	ini_set_entry(settings_file, session, "text_dest", "text_dest");
+	if (path_set == NULL) is_set = g_key_file_get_boolean(settings_file, session, "is_set", NULL);
 
-	ini_set_entry(settings_file, session, "text_notes", "entry_notes");
-	ini_set_entry(settings_file, session, "text_com_before", "entry_com_before");
-	ini_set_entry(settings_file, session, "text_com_after", "entry_com_after");
+	if (is_set && path_set == NULL) {
+		GtkTreeIter iter;
+		gboolean enabled;
+		gint i = 0;
+		GtkTreePath *path = gtk_tree_path_new_first();
 
-	stmp = g_key_file_get_string(settings_file, session, "text_addit", NULL);
-	if (stmp == NULL) stmp = g_strconcat("", NULL);
-	gtk_text_buffer_set_text(gtk_text_view_get_buffer((GtkTextView*) gtk_builder_get_object(builder, "textview_additional")), stmp, -1);
-	g_free(stmp);
+		while (gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path)) {
+			stmp = g_strconcat("set_session_enabled_", groups[i], NULL);
+			enabled = g_key_file_get_boolean(settings_file, session, stmp, NULL);
+			g_free(stmp);
 
-	set_checkbox("check_time", g_key_file_get_boolean(settings_file, session, "check_time", NULL));
-	set_checkbox("check_perm", g_key_file_get_boolean(settings_file, session, "check_perm", NULL));
-	set_checkbox("check_owner", g_key_file_get_boolean(settings_file, session, "check_owner", NULL));
-	set_checkbox("check_group", g_key_file_get_boolean(settings_file, session, "check_group", NULL));
-	set_checkbox("check_onefs", g_key_file_get_boolean(settings_file, session, "check_onefs", NULL));
-	set_checkbox("check_verbose", g_key_file_get_boolean(settings_file, session, "check_verbose", NULL));
-	set_checkbox("check_progr", g_key_file_get_boolean(settings_file, session, "check_progr", NULL));
-	set_checkbox("check_delete", g_key_file_get_boolean(settings_file, session, "check_delete", NULL));
-	set_checkbox("check_exist", g_key_file_get_boolean(settings_file, session, "check_exist", NULL));
-	set_checkbox("check_size", g_key_file_get_boolean(settings_file, session, "check_size", NULL));
-	set_checkbox("check_skipnew", g_key_file_get_boolean(settings_file, session, "check_skipnew", NULL));
-	set_checkbox("check_windows", g_key_file_get_boolean(settings_file, session, "check_windows", NULL));
+			if (enabled) gtk_list_store_set(liststore_session, &iter, 1, TRUE, -1);
+			gtk_tree_path_next(path);
+			i++;
+		}
+		gtk_tree_path_free(path);
+	} else {
+		ini_set_entry(settings_file, session, "text_source", "text_source");
+		ini_set_entry(settings_file, session, "text_dest", "text_dest");
 
-	set_checkbox("check_sum", g_key_file_get_boolean(settings_file, session, "check_sum", NULL));
-	set_checkbox("check_symlink", g_key_file_get_boolean(settings_file, session, "check_symlink", NULL));
-	set_checkbox("check_hardlink", g_key_file_get_boolean(settings_file, session, "check_hardlink", NULL));
-	set_checkbox("check_dev", g_key_file_get_boolean(settings_file, session, "check_dev", NULL));
-	set_checkbox("check_update", g_key_file_get_boolean(settings_file, session, "check_update", NULL));
-	set_checkbox("check_keepart", g_key_file_get_boolean(settings_file, session, "check_keepart", NULL));
-	set_checkbox("check_mapuser", g_key_file_get_boolean(settings_file, session, "check_mapuser", NULL));
-	set_checkbox("check_compr", g_key_file_get_boolean(settings_file, session, "check_compr", NULL));
-	set_checkbox("check_backup", g_key_file_get_boolean(settings_file, session, "check_backup", NULL));
-	set_checkbox("check_itemized", g_key_file_get_boolean(settings_file, session, "check_itemized", NULL));
-	set_checkbox("check_norecur", g_key_file_get_boolean(settings_file, session, "check_norecur", NULL));
-	set_checkbox("check_protectargs", g_key_file_get_boolean(settings_file, session, "check_protectargs", NULL));
+		ini_set_entry(settings_file, session, "text_notes", "entry_notes");
+		ini_set_entry(settings_file, session, "text_com_before", "entry_com_before");
+		ini_set_entry(settings_file, session, "text_com_after", "entry_com_after");
 
-	set_checkbox("check_com_before", g_key_file_get_boolean(settings_file, session, "check_com_before", NULL));
-	set_checkbox("check_com_halt", g_key_file_get_boolean(settings_file, session, "check_com_halt", NULL));
-	set_checkbox("check_com_after", g_key_file_get_boolean(settings_file, session, "check_com_after", NULL));
-	set_checkbox("check_com_onerror", g_key_file_get_boolean(settings_file, session, "check_com_onerror", NULL));
-	set_checkbox("check_browse_files", g_key_file_get_boolean(settings_file, session, "check_browse_files", NULL));
-#ifndef MAEMO
-	set_checkbox("check_superuser", g_key_file_get_boolean(settings_file, session, "check_superuser", NULL));
-#endif
+		stmp = g_key_file_get_string(settings_file, session, "text_addit", NULL);
+		if (stmp == NULL) stmp = g_strconcat("", NULL);
+		gtk_text_buffer_set_text(gtk_text_view_get_buffer((GtkTextView*) gtk_builder_get_object(builder, "textview_additional")), stmp, -1);
+		g_free(stmp);
+
+		set_checkbox("check_time", g_key_file_get_boolean(settings_file, session, "check_time", NULL));
+		set_checkbox("check_perm", g_key_file_get_boolean(settings_file, session, "check_perm", NULL));
+		set_checkbox("check_owner", g_key_file_get_boolean(settings_file, session, "check_owner", NULL));
+		set_checkbox("check_group", g_key_file_get_boolean(settings_file, session, "check_group", NULL));
+		set_checkbox("check_onefs", g_key_file_get_boolean(settings_file, session, "check_onefs", NULL));
+		set_checkbox("check_verbose", g_key_file_get_boolean(settings_file, session, "check_verbose", NULL));
+		set_checkbox("check_progr", g_key_file_get_boolean(settings_file, session, "check_progr", NULL));
+		set_checkbox("check_delete", g_key_file_get_boolean(settings_file, session, "check_delete", NULL));
+		set_checkbox("check_exist", g_key_file_get_boolean(settings_file, session, "check_exist", NULL));
+		set_checkbox("check_size", g_key_file_get_boolean(settings_file, session, "check_size", NULL));
+		set_checkbox("check_skipnew", g_key_file_get_boolean(settings_file, session, "check_skipnew", NULL));
+		set_checkbox("check_windows", g_key_file_get_boolean(settings_file, session, "check_windows", NULL));
+
+		set_checkbox("check_sum", g_key_file_get_boolean(settings_file, session, "check_sum", NULL));
+		set_checkbox("check_symlink", g_key_file_get_boolean(settings_file, session, "check_symlink", NULL));
+		set_checkbox("check_hardlink", g_key_file_get_boolean(settings_file, session, "check_hardlink", NULL));
+		set_checkbox("check_dev", g_key_file_get_boolean(settings_file, session, "check_dev", NULL));
+		set_checkbox("check_update", g_key_file_get_boolean(settings_file, session, "check_update", NULL));
+		set_checkbox("check_keepart", g_key_file_get_boolean(settings_file, session, "check_keepart", NULL));
+		set_checkbox("check_mapuser", g_key_file_get_boolean(settings_file, session, "check_mapuser", NULL));
+		set_checkbox("check_compr", g_key_file_get_boolean(settings_file, session, "check_compr", NULL));
+		set_checkbox("check_backup", g_key_file_get_boolean(settings_file, session, "check_backup", NULL));
+		set_checkbox("check_itemized", g_key_file_get_boolean(settings_file, session, "check_itemized", NULL));
+		set_checkbox("check_norecur", g_key_file_get_boolean(settings_file, session, "check_norecur", NULL));
+		set_checkbox("check_protectargs", g_key_file_get_boolean(settings_file, session, "check_protectargs", NULL));
+
+		set_checkbox("check_com_before", g_key_file_get_boolean(settings_file, session, "check_com_before", NULL));
+		set_checkbox("check_com_halt", g_key_file_get_boolean(settings_file, session, "check_com_halt", NULL));
+		set_checkbox("check_com_after", g_key_file_get_boolean(settings_file, session, "check_com_after", NULL));
+		set_checkbox("check_com_onerror", g_key_file_get_boolean(settings_file, session, "check_com_onerror", NULL));
+		set_checkbox("check_browse_files", g_key_file_get_boolean(settings_file, session, "check_browse_files", NULL));
+	#ifndef MAEMO
+		set_checkbox("check_superuser", g_key_file_get_boolean(settings_file, session, "check_superuser", NULL));
+	#endif
+	}
 
 	g_key_file_free (settings_file);
+
+	GtkWidget *widget = (GtkWidget*) gtk_builder_get_object(builder, "table_set");
+	is_set ? gtk_widget_show(widget) : gtk_widget_hide(widget);
+	widget = (GtkWidget*) gtk_builder_get_object(builder, "table_basic");
+	is_set ? gtk_widget_hide(widget) : gtk_widget_show(widget);
+	widget = (GtkWidget*) gtk_builder_get_object(builder, "table_advanced");
+	is_set ? gtk_widget_hide(widget) : gtk_widget_show(widget);
+	widget = (GtkWidget*) gtk_builder_get_object(builder, "table_extra");
+	is_set ? gtk_widget_hide(widget) : gtk_widget_show(widget);
 }
 
 
@@ -256,54 +290,80 @@ void save_settings(gchar *session, gchar *filename) {
 	}
 
 	// save specific session
-	g_key_file_set_string(settings_file, session, "text_source", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "text_source")));
-	g_key_file_set_string(settings_file, session, "text_dest", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "text_dest")));
-	g_key_file_set_string(settings_file, session, "text_notes", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_notes")));
-	g_key_file_set_string(settings_file, session, "text_com_before", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_com_before")));
-	g_key_file_set_string(settings_file, session, "text_com_after", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_com_after")));
 
-	GtkTextIter iter1, iter2;
-	GtkTextBuffer *buf = gtk_text_view_get_buffer((GtkTextView*) gtk_builder_get_object(builder, "textview_additional"));
-	gtk_text_buffer_get_start_iter(buf, &iter1);
-	gtk_text_buffer_get_end_iter(buf, &iter2);
-	stmp = gtk_text_buffer_get_text(buf, &iter1, &iter2, FALSE);
-	g_key_file_set_string(settings_file, session, "text_addit", stmp);
-	g_free(stmp);
+	if (is_set) {
+		GtkTreeIter iter;
+		gboolean enabled, cur_is_set;
+		gint i = 0;
+		GtkTreePath *path = gtk_tree_path_new_first();
+		g_key_file_remove_group(settings_file, session, NULL);		// empties the group before adding
+		g_key_file_set_boolean(settings_file, session, "is_set", TRUE);
 
-	g_key_file_set_boolean(settings_file, session, "check_time", get_checkbox("check_time"));
-	g_key_file_set_boolean(settings_file, session, "check_perm", get_checkbox("check_perm"));
-	g_key_file_set_boolean(settings_file, session, "check_owner", get_checkbox("check_owner"));
-	g_key_file_set_boolean(settings_file, session, "check_group", get_checkbox("check_group"));
-	g_key_file_set_boolean(settings_file, session, "check_onefs", get_checkbox("check_onefs"));
-	g_key_file_set_boolean(settings_file, session, "check_verbose", get_checkbox("check_verbose"));
-	g_key_file_set_boolean(settings_file, session, "check_progr", get_checkbox("check_progr"));
-	g_key_file_set_boolean(settings_file, session, "check_delete", get_checkbox("check_delete"));
-	g_key_file_set_boolean(settings_file, session, "check_exist", get_checkbox("check_exist"));
-	g_key_file_set_boolean(settings_file, session, "check_size", get_checkbox("check_size"));
-	g_key_file_set_boolean(settings_file, session, "check_skipnew", get_checkbox("check_skipnew"));
-	g_key_file_set_boolean(settings_file, session, "check_windows", get_checkbox("check_windows"));
+		while (gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path)) {
+			gtk_tree_model_get((GtkTreeModel*) liststore_session, &iter, 2, &cur_is_set, -1);
+			if (cur_is_set) {
+				gtk_tree_model_get((GtkTreeModel*) liststore_session, &iter, 1, &enabled, -1);
+				if (enabled) {
+					stmp = g_strconcat("set_session_enabled_", groups[i], NULL);
+					g_key_file_set_boolean(settings_file, session, stmp, TRUE);
+					g_free(stmp);
+				}
+			}
+			gtk_tree_path_next(path);
+			i++;
+		}
+		gtk_tree_path_free(path);
+	} else {
+		g_key_file_set_boolean(settings_file, session, "is_set", FALSE);
+		g_key_file_set_string(settings_file, session, "text_source", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "text_source")));
+		g_key_file_set_string(settings_file, session, "text_dest", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "text_dest")));
+		g_key_file_set_string(settings_file, session, "text_notes", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_notes")));
+		g_key_file_set_string(settings_file, session, "text_com_before", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_com_before")));
+		g_key_file_set_string(settings_file, session, "text_com_after", gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_com_after")));
 
-	g_key_file_set_boolean(settings_file, session, "check_sum", get_checkbox("check_sum"));
-	g_key_file_set_boolean(settings_file, session, "check_symlink", get_checkbox("check_symlink"));
-	g_key_file_set_boolean(settings_file, session, "check_hardlink", get_checkbox("check_hardlink"));
-	g_key_file_set_boolean(settings_file, session, "check_dev", get_checkbox("check_dev"));
-	g_key_file_set_boolean(settings_file, session, "check_update", get_checkbox("check_update"));
-	g_key_file_set_boolean(settings_file, session, "check_keepart", get_checkbox("check_keepart"));
-	g_key_file_set_boolean(settings_file, session, "check_mapuser", get_checkbox("check_mapuser"));
-	g_key_file_set_boolean(settings_file, session, "check_compr", get_checkbox("check_compr"));
-	g_key_file_set_boolean(settings_file, session, "check_backup", get_checkbox("check_backup"));
-	g_key_file_set_boolean(settings_file, session, "check_itemized", get_checkbox("check_itemized"));
-	g_key_file_set_boolean(settings_file, session, "check_norecur", get_checkbox("check_norecur"));
-	g_key_file_set_boolean(settings_file, session, "check_protectargs", get_checkbox("check_protectargs"));
+		GtkTextIter iter1, iter2;
+		GtkTextBuffer *buf = gtk_text_view_get_buffer((GtkTextView*) gtk_builder_get_object(builder, "textview_additional"));
+		gtk_text_buffer_get_start_iter(buf, &iter1);
+		gtk_text_buffer_get_end_iter(buf, &iter2);
+		stmp = gtk_text_buffer_get_text(buf, &iter1, &iter2, FALSE);
+		g_key_file_set_string(settings_file, session, "text_addit", stmp);
+		g_free(stmp);
 
-	g_key_file_set_boolean(settings_file, session, "check_com_before", get_checkbox("check_com_before"));
-	g_key_file_set_boolean(settings_file, session, "check_com_halt", get_checkbox("check_com_halt"));
-	g_key_file_set_boolean(settings_file, session, "check_com_after", get_checkbox("check_com_after"));
-	g_key_file_set_boolean(settings_file, session, "check_com_onerror", get_checkbox("check_com_onerror"));
-	g_key_file_set_boolean(settings_file, session, "check_browse_files", get_checkbox("check_browse_files"));
-#ifndef MAEMO
-	g_key_file_set_boolean(settings_file, session, "check_superuser", get_checkbox("check_superuser"));
-#endif
+		g_key_file_set_boolean(settings_file, session, "check_time", get_checkbox("check_time"));
+		g_key_file_set_boolean(settings_file, session, "check_perm", get_checkbox("check_perm"));
+		g_key_file_set_boolean(settings_file, session, "check_owner", get_checkbox("check_owner"));
+		g_key_file_set_boolean(settings_file, session, "check_group", get_checkbox("check_group"));
+		g_key_file_set_boolean(settings_file, session, "check_onefs", get_checkbox("check_onefs"));
+		g_key_file_set_boolean(settings_file, session, "check_verbose", get_checkbox("check_verbose"));
+		g_key_file_set_boolean(settings_file, session, "check_progr", get_checkbox("check_progr"));
+		g_key_file_set_boolean(settings_file, session, "check_delete", get_checkbox("check_delete"));
+		g_key_file_set_boolean(settings_file, session, "check_exist", get_checkbox("check_exist"));
+		g_key_file_set_boolean(settings_file, session, "check_size", get_checkbox("check_size"));
+		g_key_file_set_boolean(settings_file, session, "check_skipnew", get_checkbox("check_skipnew"));
+		g_key_file_set_boolean(settings_file, session, "check_windows", get_checkbox("check_windows"));
+
+		g_key_file_set_boolean(settings_file, session, "check_sum", get_checkbox("check_sum"));
+		g_key_file_set_boolean(settings_file, session, "check_symlink", get_checkbox("check_symlink"));
+		g_key_file_set_boolean(settings_file, session, "check_hardlink", get_checkbox("check_hardlink"));
+		g_key_file_set_boolean(settings_file, session, "check_dev", get_checkbox("check_dev"));
+		g_key_file_set_boolean(settings_file, session, "check_update", get_checkbox("check_update"));
+		g_key_file_set_boolean(settings_file, session, "check_keepart", get_checkbox("check_keepart"));
+		g_key_file_set_boolean(settings_file, session, "check_mapuser", get_checkbox("check_mapuser"));
+		g_key_file_set_boolean(settings_file, session, "check_compr", get_checkbox("check_compr"));
+		g_key_file_set_boolean(settings_file, session, "check_backup", get_checkbox("check_backup"));
+		g_key_file_set_boolean(settings_file, session, "check_itemized", get_checkbox("check_itemized"));
+		g_key_file_set_boolean(settings_file, session, "check_norecur", get_checkbox("check_norecur"));
+		g_key_file_set_boolean(settings_file, session, "check_protectargs", get_checkbox("check_protectargs"));
+
+		g_key_file_set_boolean(settings_file, session, "check_com_before", get_checkbox("check_com_before"));
+		g_key_file_set_boolean(settings_file, session, "check_com_halt", get_checkbox("check_com_halt"));
+		g_key_file_set_boolean(settings_file, session, "check_com_after", get_checkbox("check_com_after"));
+		g_key_file_set_boolean(settings_file, session, "check_com_onerror", get_checkbox("check_com_onerror"));
+		g_key_file_set_boolean(settings_file, session, "check_browse_files", get_checkbox("check_browse_files"));
+		#ifndef MAEMO
+		g_key_file_set_boolean(settings_file, session, "check_superuser", get_checkbox("check_superuser"));
+		#endif
+	}
 
 	key_data = g_key_file_to_data(settings_file, NULL, &error_handler);
 	key_file = fopen(settings_file_name, "w");
@@ -332,11 +392,14 @@ gboolean load_groups(gchar *session) {
 		GError *error_handler = NULL;
 		GKeyFile *settings_file;
 		GtkComboBox *combo;
+		GtkTreeIter iter;
+		GtkTreeModelFilter *filter = (GtkTreeModelFilter*) gtk_builder_get_object(builder, "treemodelfilter_set");
 		gint i;
 		gboolean flag = FALSE;
 
 		combo = (GtkComboBox*) gtk_builder_get_object(builder, "combo_session");
-		for (i = 0; i < session_number; i++) gtk_combo_box_remove_text(combo, 0);
+		gtk_list_store_clear(liststore_session);
+		// for (i = 0; i < session_number; i++) gtk_combo_box_remove_text(combo, 0);
 
 		settings_file = g_key_file_new();
 		g_key_file_load_from_file(settings_file, settings_file_path, 	G_KEY_FILE_NONE, &error_handler);
@@ -368,16 +431,21 @@ gboolean load_groups(gchar *session) {
 
 		i = 0;
 		while (groups[i] != NULL) {
-			gtk_combo_box_append_text(combo, groups[i]);
+			// gtk_combo_box_append_text(combo, groups[i]);
+			gtk_list_store_append(liststore_session, &iter);
+			gtk_list_store_set(liststore_session, &iter, 0, groups[i], 2, !g_key_file_get_boolean(settings_file, groups[i], "is_set", NULL), -1);
+
 			if (strcmp(groups[i], session) == 0) {
 				gtk_combo_box_set_active(combo, i);
 				session_last = i;
 			}
 			i++;
 		}
+		gtk_tree_model_filter_set_visible_column(filter, 2);
+		gtk_tree_model_filter_refilter(filter);
 
-		g_key_file_free (settings_file);
-		g_clear_error (&error_handler);
+		g_key_file_free(settings_file);
+		g_clear_error(&error_handler);
 		if (session_last == -1) return FALSE;
 	}
 	return TRUE;
@@ -528,8 +596,44 @@ void build_rsync_cmd() {
 }
 
 
-void on_play_clicked(GtkButton *button, gpointer user_data) {
+gboolean set_next() {
+	static gint i = -1;
+	gboolean enabled, more;
+	GtkTreeIter iter;
+
+	if (i == -1) {
+		session_set = session_last;
+		path_set = gtk_tree_path_new_first();
+		i = 0;
+	}
+
+	while ((more = gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path_set))) {
+		gtk_tree_model_get((GtkTreeModel*) liststore_session, &iter, 1, &enabled, -1);
+		gtk_tree_path_next(path_set);
+		i++;
+		if (enabled) {
+			session_last = i - 1;
+			load_settings(groups[session_last], NULL);
+			break;
+		}
+	}
+
+	if (!more) {
+		session_last = session_set;
+		session_set = -1;
+		gtk_tree_path_free(path_set);
+		path_set = NULL;
+		i = -1;
+	}
+	return more;
+}
+
+
+gboolean on_play_clicked(GtkButton *button, gpointer user_data) {
 	gchar *gtmp;
+	gboolean more = FALSE;
+
+	if (is_set) more = set_next();
 
 	build_rsync_cmd();
 	paused = FALSE;
@@ -547,7 +651,7 @@ void on_play_clicked(GtkButton *button, gpointer user_data) {
 		g_free(gtmp);
 	}
 
-	gtk_window_set_title((GtkWindow*) rsync_window, "rsync: running");
+	gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: running"));
 	gtk_button_set_label((GtkButton*) gtk_builder_get_object(builder, "close"), "gtk-stop");
 	gtk_button_set_label((GtkButton*) gtk_builder_get_object(builder, "button_pause"), "gtk-media-pause");
 	gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), TRUE);
@@ -557,8 +661,8 @@ void on_play_clicked(GtkButton *button, gpointer user_data) {
 	set_file_progress(0);
 	set_global_progress(0);
 	gtk_label_set_text((GtkLabel*) gtk_builder_get_object(builder, "label_file"), dryrunning ? _("Simulation in progress") : "");
-	set_trayicon_icon(TRUE);
 	gtk_widget_show(rsync_window);
+	return more;
 }
 
 
@@ -692,7 +796,8 @@ void on_close_clicked(GtkButton *button, gpointer user_data) {
 		kill(rsync_pid, SIGTERM);
 		gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), FALSE);
 	} else {
-		gtk_widget_hide(rsync_window);
+		if (on_play_clicked(NULL, NULL)) on_rsync_show(NULL, NULL);
+		else gtk_widget_hide(rsync_window);
 	}
 	if (cmdline_execute) on_main_destroy((GtkObject*) main_window, NULL);
 }
@@ -753,7 +858,7 @@ void rsync_cleanup(gpointer data) {
 	set_global_progress(1);
 	gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), FALSE);
 	gtk_widget_set_sensitive((GtkWidget*) view, TRUE);
-	gtk_window_set_title((GtkWindow*) rsync_window, "rsync: stopped");
+	gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: stopped"));
 
 	g_io_channel_unref(chout);
 	g_io_channel_unref(cherr);
@@ -767,7 +872,6 @@ void rsync_cleanup(gpointer data) {
 	if (config_errorlist && had_error) on_button_show_errors_clicked(NULL, NULL);
 	else gtk_window_set_urgency_hint((GtkWindow*) rsync_window, TRUE);
 	set_trayicon_icon(FALSE);
-	dryrunning = FALSE;
 	if (cmdline_execute && !(config_errorlist && had_error) && !cmdline_stayopen) on_close_clicked(NULL, NULL);
 
 	if (log_file != NULL) {
@@ -932,11 +1036,11 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 
 	rsync_pid = 0;
 	had_error = FALSE;
-	gtk_label_set_ellipsize((GtkLabel*)gtk_builder_get_object(builder, "label_file"), PANGO_ELLIPSIZE_MIDDLE);
+	gtk_label_set_ellipsize((GtkLabel*) gtk_builder_get_object(builder, "label_file"), PANGO_ELLIPSIZE_MIDDLE);
 
 	if (get_checkbox("check_com_before")) {
 		comm_str = (gchar*) gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_com_before"));
-		if (!g_spawn_command_line_sync(comm_str, &comm_out, &comm_err, NULL, NULL)) {
+		if (!g_spawn_command_line_sync(comm_str, &comm_out, &comm_err, &err, NULL) || err) {
 			comm_out = g_strconcat("", NULL);
 			comm_err = g_strconcat("Error launching command!\n", NULL);
 			if (get_checkbox("check_com_halt")) halt = TRUE;
@@ -967,6 +1071,7 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	g_get_current_time(&startime);
 
 	if (!halt && g_spawn_async_with_pipes(NULL, argv, NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &rsync_pid, NULL, &out, &err, NULL)) {
+		set_trayicon_icon(TRUE);
 		chout = g_io_channel_unix_new(out);
 		cherr = g_io_channel_unix_new(err);
 		g_io_channel_set_flags(chout, G_IO_FLAG_NONBLOCK, NULL);
@@ -986,6 +1091,7 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	} else {
 		gtk_button_set_label((GtkButton*) gtk_builder_get_object(builder, "close"), "gtk-close");
 		gtk_widget_set_sensitive((GtkWidget*) gtk_builder_get_object(builder, "button_pause"), FALSE);
+		gtk_widget_set_sensitive((GtkWidget*) view, TRUE);
 		if (!halt) {
 #ifndef MAEMO
 			error = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CANCEL, _("Cannot execute rsync"));
@@ -1005,6 +1111,7 @@ void on_rsync_hide(GtkObject *object, gpointer user_data) {
 		if (paused) kill(rsync_pid, SIGCONT);
 		kill(rsync_pid, SIGTERM);
 	}
+	dryrunning = FALSE;
 }
 
 
@@ -1030,8 +1137,9 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 	if (result == GTK_RESPONSE_OK) {
 		gchar *newses;
 		newses = (gchar*) gtk_entry_get_text((GtkEntry*) gtk_builder_get_object(builder, "entry_session_name"));
+		is_set = get_checkbox("checkbutton_isset");
 
-		if (strcmp(CONFIG_GROUP, newses) == 0) {
+		if (strcmp(CONFIG_GROUP, newses) == 0 || strcmp("", newses) == 0) {
 #ifndef MAEMO
 			confirm = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_CANCEL, _("This session name is reserved, you cannot call it like this"));
@@ -1151,6 +1259,8 @@ void on_button_about_clicked(GtkButton *button, gpointer user_data) {
 		_("Czech translation by Martin Balajka <m.balajka@gmail.com>"),
 		_("Galician translation by Daniel Espinheira <daniel.espinheira@gmail.com>"),
 		_("Catalan translation by Josep Sànchez <josep.sanchez@ubuntu.cat>"),
+		_("Brazilian Portuguese translation by Fábio Nogueira <deb-user-ba@ubuntu.com>"),
+		_("Indonesian translation by Waluyo Adi Siswanto <was.uthm@gmail.com>"),
 		NULL
 	};
 
@@ -1178,14 +1288,14 @@ void on_button_pause_clicked(GtkButton *button, gpointer user_data) {
 	if (paused) {
 		if (rsync_pid) kill(rsync_pid, SIGCONT);
 		gtk_button_set_label(tmpbutt, "gtk-media-pause");
-		gtk_window_set_title((GtkWindow*) rsync_window, "rsync: running");
+		gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: running"));
 		g_get_current_time(&tmptime);
 		startime.tv_sec += tmptime.tv_sec - pausedtime.tv_sec;
 		startime.tv_usec += tmptime.tv_usec - pausedtime.tv_usec;
 	} else {
 		if (rsync_pid) kill(rsync_pid, SIGSTOP);
 		gtk_button_set_label(tmpbutt, "gtk-media-play");
-		gtk_window_set_title((GtkWindow*) rsync_window, "rsync: paused");
+		gtk_window_set_title((GtkWindow*) rsync_window, _("rsync: paused"));
 		g_get_current_time(&pausedtime);
 		g_idle_add(scroll_idle, gtk_builder_get_object(builder, "textview_output"));
 	}
@@ -1404,3 +1514,15 @@ void on_contribute_activate(GtkMenuItem *menuitem, gpointer user_data) {
 #endif
 }
 
+
+void on_cellrenderertoggle1_toggled(GtkCellRendererToggle *cell, gchar *path_str, gpointer data) {
+	GtkTreeIter iter;
+	gboolean enabled;
+	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
+
+	gtk_tree_model_get_iter((GtkTreeModel*) liststore_session, &iter, path);
+	gtk_tree_path_free(path);
+	gtk_tree_model_get((GtkTreeModel*) liststore_session, &iter, 1, &enabled, -1);
+	enabled ^= 1;
+	gtk_list_store_set(liststore_session, &iter, 1, enabled, -1);
+}
